/* util
 * @Author: Kvkens(yueming@yonyou.com)
 * @Date:   2019-01-21 12:59:30
 * @Last Modified by:   Kvkens
 * @Last Modified time: 2019-05-10 10:16:46
 */

const path = require("path");
const chalk = require("chalk");
const fse = require("fs-extra");
const glob = require("glob");
const { defaultPluginOptions, pluginsMap } = require("../webpack/default");
let config = null;

/**
 * 获得当前运行路径
 * @param {string} file 获得文件路径
 */
exports.getRunPath = (file) => {
    return path.resolve(".", file);
};
/**
 * 获得uba.config
 */
exports.getUcfConfig = (cmd) => {
    try {
        let isUcfConfig = fse.pathExistsSync(this.getRunPath("ucf.config.js"));
        const ynfConfigPath = glob.sync(`**/ynf-dx-scripts/ucf.config.js`)?.[0]
        const isYnfUcfConfig = !!glob.sync(`**/ynf-dx-scripts/ucf.config.js`)?.[0]
        // 缓存配置文件，防止二次修改无效

        if (config == null) {
            // 兼容mtl.config
            config = require(this.getRunPath(isUcfConfig ? "ucf.config.js" : isYnfUcfConfig ? ynfConfigPath : "mtl.config.js"))(
                cmd[0] == "start" ? "development" : "production",
                cmd
            );
        }
        return config;
    } catch (error) {
        this.errorLog(error, 'The "ucf.config.js" configuration file was not found', true);
        process.exit(0);
    }
};

/**
 * 获取package.json
 */
exports.getPkg = () => {
    return require("../package.json");
};

/**
 * 错误处理
 * @param {*} err 错误对象
 * @param {*} msg 描述文字
 * @param {*} statck 是否显示错误栈
 */
exports.errorLog = (err, msg, statck) => {
    console.log(chalk.bold.red(`>> Error Message:`));
    console.log(chalk.cyan(err.message));
    msg && console.log(chalk.yellow(`[message] ${msg}`));
    console.log(chalk.bold.red(`>> End`));
    statck && console.log();
    statck && console.log(chalk.bold.red(`>> Stack Message:`));
    statck && console.log(chalk.cyan(err.stack));
    statck && console.log(chalk.bold.red(`>> End`));
};

/* UCF handle param '-- set-cookie'
 * @Author: limengag
 * @Date:   2023-05-23
 */
exports.writeToken = (element) => {
    if (process.argv.filter((e) => e.startsWith("set-token")).length > 0) {
        element.headers.YYCtoken = "asd";
    }
};

/* UCF handle param '-- set-cookie'
 * @Author: limengag
 * @Date:   2023-05-23
 */
exports.writeCookie = (element) => {
    if (process.argv.filter((e) => e.startsWith("set-cookie=")).length > 0) {
        let argvCookie = process.argv.filter((e) => e.startsWith("set-cookie="))[0].substring(11);
        if (argvCookie.endsWith(";")) {
            argvCookie = argvCookie.substring(0, argvCookie.length - 1);
        }
        const argvCookieArray = argvCookie.split(";");
        const headersCookieArray =
            element?.headers?.cookie != undefined ? element.headers.cookie.split(";") : [];
        headersCookieArray.forEach((e) => {
            if (e.indexOf("=") > -1) {
                const name_ = e.substring(0, e.indexOf("=") + 1).trim();
                const exist = argvCookieArray.some((ele) => ele.trim().startsWith(name_));
                if (!exist) {
                    argvCookieArray.push(e);
                }
            }
        });
        let argvCookieNew = argvCookieArray.join(";");
        if (!argvCookieNew.endsWith(";")) {
            argvCookieNew += ";";
        }
        element.headers.cookie = argvCookieNew;
    }
};

const getType = function (variable) {
    return Object.prototype.toString.call(variable).slice(8, -1).toLocaleLowerCase();
};

/**
 * isType
 * @param {*} variable
 * @returns
 */
const isType = function (variable, type) {
    return getType(variable) === type;
};

//扩展
exports.defaultExtendFunc = (userConfig, defaultConfig) => {
    //判断ucf.config传入变量类型
    let userConfigType = typeof userConfig;
    if (userConfigType === "function") {
        let func = userConfig;
        return func(defaultConfig);
    }
    if (getType(userConfig) == getType(defaultConfig)) {
        if (Array.isArray(userConfig)) {
            return [...defaultConfig, ...userConfig];
        }
        if (userConfigType === "object") {
            return { ...defaultConfig, ...userConfig };
        }
    }
    return userConfig || defaultConfig;
};

//扩展
exports.htmlWebpackPluginHook = (cfg, mode) => {
    const htmlConfList = [];
    //启动器控制
    const _bootList = new Set();
    let { bootName, scan_root = "ucf-apps", static_url, bootList = true } = cfg;
    let isProd = mode === "prod";
    // 入口文件名支持用户传入(统一名称), 如接入tns 必需的参数
    if (!bootName) {
        console.log(chalk.green(`启动默认入口app.js`));
        bootName = "app.js";
    }
    const bootFiles = glob.sync(`./${scan_root}/**/src/${bootName}`);
    if (!bootFiles.length) {
        console.log(chalk.red(`找不到入口文件${bootName}，检查入口文件是否被设置正确`));
        process.exit();
    }
    let _context = "";
    //TODO 当publicPath=true 因为start.js webpack-dev-middleware 中配置了publicPath, 所以此处产出路径不在包含 context
    if (!cfg.publicPath && cfg.context) {
        _context = `${cfg.context}/`;
    }


    //构造模块加载入口以及html出口
    bootFiles.forEach((_path) => {
        //模块名
        const module = _path.split(`./${scan_root}/`)[1].split(`/src/${bootName}`)[0];
        const chunk = `${_context}${module}/index`;
        const targetDir = _path.split(`/${bootName}`)[0];
        // 兼容其他类型的模版配置，如：ejs
        const templateType = cfg.templateType || "html";
        const htmlConf = {
            filename: `${chunk}.html`,
            template: `${targetDir}/index.${templateType}`,
            inject: "body",
            chunks: isProd ? ["vendor", chunk] : [chunk],
            hash: true,
            static_url: static_url ? static_url : "",
        };
        let entresChunks = isProd
            ? _path
            : [_path, require.resolve("./webpack-hot-middleware/client")];
        //处理启动器逻辑
        if (bootList && typeof bootList == "boolean") {
            htmlConfList.push({
                entryKey: chunk,
                entryChunk: entresChunks,
                configOption: htmlConf,
                moduleName: module,
            });
        } else if (Array.isArray(bootList) && bootList.length > 0) {
            bootList.forEach((item) => {
                _bootList.add(item);
            });
            if (_bootList.has(module)) {
                htmlConfList.push({
                    entryKey: chunk,
                    entryChunk: entresChunks,
                    configOption: htmlConf,
                    moduleName: module,
                });
            }
        }
    });
    return htmlConfList;
};

exports.configHandler = (func, config) => {
    if (typeof func === "function") {
        return func(config) || config;
    } else {
        if (isType(func, "object")) {
            return func;
        }
        return config;
    }
};

//处理传递参数
//option=>object
// {
//     hashNum:8,0代表不开启hash
//     ext:"js"
// }
exports.handleChangeOptions = (type, option) => {
    let res = null;
    let { cfgOpt, key, ext = "" } = option;
    let isObject = isType(cfgOpt, "object");
    switch (type) {
        case "hash":
            if (isObject || cfgOpt) {
                let num = cfgOpt[key] || 8;
                res = `[name].[hash:${num}].${ext}`;
            } else {
                res = `[name].${ext}`;
            }
            break;
        case "limit":
            if (isType(cfgOpt, "number")) {
                res = cfgOpt;
            } else if (isObject) {
                res = cfgOpt[key];
            } else {
                res = cfgOpt === undefined ? 8196 : 8196000;
            }
            break;
        case "publicPath":
            if (cfgOpt == "/") {
                res = "/";
            } else if (isObject) {
                res = "/" + cfgOpt[key] + "/";
                if(cfgOpt[key]){
                    res = "/" + cfgOpt[key] + "/";
                }else{
                    res = undefined;
                }
            } else if (cfgOpt && isType(cfgOpt, "string")) {
                res = "/" + cfgOpt + "/" + ext;
            } else {
                res = undefined;
            }
            res = removeExtraSlash(res)
        default:
            break;
    }
    return res;
};

function removeExtraSlash(str) {
    if(typeof(str) !=='string'){
        return str
    }
    // 判断字符串前后是否有两个斜杠
    if (str.startsWith("//")) {
      // 删除前面的一个斜杠
      str = str.substring(1);
    }
    if (str.endsWith("//")) {
      // 删除后面的一个斜杠
      str = str.substring(0, str.length - 1);
    }
    return str;
}

function newPlugin(pluginName, pluginOption) {
    const PluginClass = pluginsMap[pluginName];
    if (!PluginClass) {
        throw new Error(`Plugin ${pluginName} not found.`);
    }
    return new PluginClass(pluginOption);
}

getPlugins = (pluginOption, pluginName, htmlConfList) => {
    //多个new，比如htmlwebpackplugin
    if (isType(pluginOption, "array")) {
        // bootItem {
        //   moduleName: string; // 微应用入口名称
        //   configOption: object; // 微应用 HtmlWebPackPlugin 参数配置
        // }
        // pluginOption: BootItem[]
        let plugins = [];
        if (pluginName === "HtmlWebPackPlugin") {
            htmlConfList.forEach((confItem) => {
                let findItem = pluginOption.find((it) => it.moduleName === confItem.moduleName);
                if (findItem) {
                    plugins.push(newPlugin(pluginName, findItem.configOption));
                } else {
                    plugins.push(newPlugin(pluginName, confItem.configOption));
                }
            });
        } else {
            pluginOption.forEach((it) => {
                plugins.push(newPlugin(pluginName, it.configOption));
            });
        }
        return plugins;
    }
    if (isType(pluginOption, "object")) {
        if (pluginName === "HtmlWebPackPlugin") {
            let plugins = [];
            htmlConfList.forEach((confItem) => {
                let newOptions = {
                    ...confItem.configOption,
                    ...pluginOption,
                };
                plugins.push(newPlugin(pluginName, newOptions));
            });
            return plugins;
        } else {
            return newPlugin(pluginName, pluginOption);
        }
    }
};

exports.pluginConfigHook = (pluginConfig, pluginName, htmlConfList) => {
    let defaultConfig = defaultPluginOptions;
    let pluginOption = defaultConfig[pluginName];

    if (typeof pluginConfig === "function") {
        let func = pluginConfig;
        pluginOption = func(defaultConfig)[pluginName] || defaultConfig[pluginName];
    }

    if (isType(pluginConfig, "object")) {
        pluginOption = pluginConfig[pluginName];
        let defaultPluginOpt = defaultPluginOptions[pluginName];
        if (typeof pluginOption === "function") {
            let funcOption = pluginOption;
            pluginOption = funcOption(htmlConfList || defaultPluginOpt);
        }
    }

    if(pluginName == 'plugins'){
        return pluginOption
    }

    //没有传递参数走默认内置配置
    if (pluginConfig === undefined || pluginOption === undefined) {
        pluginOption = defaultConfig[pluginName];
        if (pluginName === "HtmlWebPackPlugin") {
            pluginOption = htmlConfList;
        }
        if (!this.isEmpty(pluginOption)) {
            return getPlugins(pluginOption, pluginName, htmlConfList);
        } else {
            return null;
        }
    }

    if (!this.isEmpty(pluginOption) || pluginName === "HtmlWebPackPlugin") {
        return getPlugins(pluginOption, pluginName, htmlConfList);
    } else {
        return null;
    }
};

exports.isEmpty = (source) => {
    if (isType(source, "array")) {
        return source.length === 0;
    }
    if (isType(source, "object")) {
        return Object.keys(source).length === 0;
    }
    return source;
};

exports.getCfg = () => {
    const argv = require("minimist")(process.argv.slice(2));
    const commands = argv;
    const cfg = this.getUcfConfig(commands._);
    return cfg;
};

//检测升级3.0版本之后的api校验
exports.checkUcfConfigOptions = (cfg) => {
    const oldOptions = [
        "css",
        "less",
        "hasHash",
        "open_source_map",
        "res_extra",
        "postcss_plugins",
        "babel_include",
        "babel_presets",
        "babel_plugins",
        "loader",
        "devPlugins",
        "buildPlugins",
        "HtmlPluginConf",
    ];
    let options = oldOptions.filter((it) => cfg[it] !== undefined);
    return options;
};


exports.getOfficialUrl = () =>{
    console.log(chalk.bgGreen("ucf-scripts官方文档地址:"));
    console.log();
    console.log('https://git.yonyou.com/yonyou-ide/tinper-next-packages/next-script/-/blob/3.x-develop/packages/ucf-scripts/README.md')
}